#include "Sherpa.H"
#include "Signal_Processes.H"
#include "Hard_Decays.H"
#include "Multiple_Interactions.H"
#include "Jet_Evolution.H"
#include "Hadronization.H"
#include "Message.H"

using namespace SHERPA;
using namespace ATOOLS;
using namespace std;

Sherpa::Sherpa() :
  p_inithandler(NULL), p_eventhandler(NULL), p_analysis(NULL)  
{
  m_errors = 0;
  m_trials = 100;
}

Sherpa::~Sherpa() 
{
  if (p_analysis)     { delete p_analysis;     p_analysis     = NULL; }
  if (p_eventhandler) { delete p_eventhandler; p_eventhandler = NULL; }
  if (p_inithandler)  { delete p_inithandler;  p_inithandler  = NULL; }
}

bool Sherpa::InitializeTheRun(int argc,char * argv[]) 
{ 
  m_path = std::string("./");
  p_inithandler  = new Initialization_Handler(argc, argv);
  int mode = p_inithandler->Mode();
  if (mode==14) {
    return PerformScan();
  }
  else {
    bool okay;
    if (p_inithandler->InitializeTheFramework()) {
      p_output = new Output_Handler(2);
      okay     =  p_inithandler->CalculateTheHardProcesses();
      if (rpa.gen.NumberOfEvents()>0)
	okay   = okay && p_inithandler->InitializeAllHardDecays();
      return okay;
    }
  }
  msg.Error()<<"Error in Sherpa::InitializeRun("<<m_path<<")"<<endl
	     <<"   Did not manage to initialize the framework."<<endl
	     <<"   Try to run nevertheless ... ."<<endl;
  return 0;
}


bool Sherpa::InitializeTheEventHandler() 
{
  p_analysis        = new Sample_Analysis(p_inithandler->Path(),p_inithandler->File());
  p_eventhandler    = new Event_Handler();
  p_eventhandler->AddEventPhase(new Signal_Processes(p_inithandler->GetMatrixElementHandler(std::string("SignalMEs")),
						     p_inithandler->GetHardDecayHandler()));
  // p_eventhandler->AddEventPhase(new Hard_Decays(p_inithandler->GetHardDecayHandler()));
  p_eventhandler->AddEventPhase(new Multiple_Interactions(p_inithandler->GetMIHandler()));
  p_eventhandler->AddEventPhase(new Analysis_Phase(p_analysis,1));
  p_eventhandler->AddEventPhase(new Jet_Evolution(p_inithandler->GetMatrixElementHandlers(),
						  p_inithandler->GetShowerHandler()));
  p_eventhandler->AddEventPhase(new Analysis_Phase(p_analysis,2));
  p_eventhandler->AddEventPhase(new Hadronization(p_inithandler->GetBeamRemnantHandler(),
  						  p_inithandler->GetFragmentationHandler()));
  p_eventhandler->AddEventPhase(new Analysis_Phase(p_analysis,3));
  return 1;
}


bool Sherpa::GenerateOneEvent() 
{
  for (int i=0;i<m_trials;i++) {
    if (p_eventhandler->GenerateEvent()) {
      if (p_output->Active()) p_output->OutputToFormat(p_eventhandler->GetBlobs());
      return 1;
    }
    m_errors++;
  }
  return 0;
}



bool Sherpa::SummarizeRun() 
{
  if (p_analysis) p_analysis->Finish("");
  return 1;
}

void Sherpa::DrawLogo() { }

bool Sherpa::PerformScan() 
{
  int np = p_inithandler->NumberOfSteps();
  for (int i=0;i<=np;++i) {
    if (p_inithandler->InitializeTheFramework(i)) {
      if (!p_inithandler->CalculateTheHardProcesses()) {
	msg.Error()<<"ERROR in Sherpa::PerfomScan("<<i<<")"<<endl;
      }
    }
    else {
      msg.Error()<<"ERROR in Sherpa::InitializeRun("<<m_path<<")"<<endl;
    }
  }

  return 1;
}
